import { useCallback, useEffect, useState } from "react";

import { AutocompleteFieldServerProps } from "../types/all-fields";
import { useDebounce } from "../utils/use-debounce";
import AutocompleteField from "./autocomplete-field";

/**
 * @description This is a server side autocomplete field that takes a server action and a function to get the options from the result of the server action.
 */
const AutocompleteFieldServer = <T, V>(
  props: AutocompleteFieldServerProps<T, V>
) => {
  const { serverAction, getOptions, ...rest } = props;
  const [options, setOptions] = useState<{ label: string; value: string }[]>(
    []
  );
  const [searchValue, setSearchValue] = useState("");
  const debouncedValue = useDebounce(searchValue, 500);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState(false);
  const serverSearch = useCallback(
    async (value: string) => {
      try {
        setLoading(true);
        const response = await serverAction(value);
        return getOptions(response);
      } catch (error) {
        console.log(error);
        setError(true);
        return [];
      } finally {
        setLoading(false);
      }
    },
    [serverAction, getOptions]
  );
  useEffect(() => {
    serverSearch(debouncedValue).then((options) => setOptions(options));
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [debouncedValue]);
  return (
    <AutocompleteField
      name={""}
      control={undefined}
      {...rest}
      serverSearch={(e) => setSearchValue(e)}
      loading={loading}
      getOptionsError={error}
      options={options}
    />
  );
};

export default AutocompleteFieldServer;
